from typing import Dict, Set


class TraceItem:
    _registry: Dict[str, "TraceItem"] = {}

    def __init__(self, id_: str, traced_to: Set["TraceItem"] = None):
        self.id = id_
        self.traced_to = traced_to or set()

    @classmethod
    def get_by_id(cls, id_):
        if id_ in cls._registry:
            item = cls._registry[id_]
        else:
            item = cls(id_)
            cls._registry[id_] = item
        return item

    @staticmethod
    def add_trace(first: "TraceItem", second: "TraceItem"):
        first.traced_to.add(second)
        second.traced_to.add(first)



from tracematrix.reporters.base_reporter import Reporter
from tracematrix.reporters.csv_reporter import CsvReporter
from tracematrix.reporters.html_reporter import HtmlReporter

__all__ = ["Reporter", "CsvReporter", "HtmlReporter"]



import csv
from typing import List

from tracematrix.item import TraceItem
from tracematrix.reporters.base_reporter import Reporter


class CsvReporter(Reporter):
    @staticmethod
    def write(
        outputfile: str, testcases: List[TraceItem], requirements: List[TraceItem]
    ) -> None:
        fieldnames = ["testcase"] + [req.id for req in requirements] + ["total"]
        with open(outputfile, "w", encoding="utf8") as outfile:
            writer = csv.DictWriter(outfile, fieldnames, delimiter=";")
            writer.writeheader()
            for tc in testcases:
                rowdict = {"testcase": tc.id}
                for traced_req in tc.traced_to:
                    rowdict[traced_req.id] = "x"
                rowdict["total"] = len(tc.traced_to)
                writer.writerow(rowdict)
            result_row = {"testcase": "total"}
            for req in requirements:
                result_row[req.id] = len(req.traced_to)
            writer.writerow(result_row)



from typing import List
from pathlib import Path

from jinja2 import Environment, FileSystemLoader, PackageLoader, select_autoescape

from tracematrix.item import TraceItem
from tracematrix.reporters.base_reporter import Reporter


class HtmlReporter(Reporter):
    env = Environment(
        loader=FileSystemLoader(Path(__file__).parent),
        autoescape=select_autoescape(["html", "xml"]),
    )
    template = env.get_template("template.html")

    @classmethod
    def write(
        cls, outputfile: str, testcases: List[TraceItem], requirements: List[TraceItem]
    ) -> None:
        with open(outputfile, "w", encoding="utf8") as outfile:
            outfile.write(
                cls.template.render(testcases=testcases, requirements=requirements)
            )



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['tracematrix', 'tracematrix.reporters']

package_data = \
{'': ['*']}

install_requires = \
['Jinja2>=3.0.1,<4.0.0']

setup_kwargs = {
    'name': 'tracematrix',
    'version': '0.2.0',
    'description': 'Tool to create a traceability matrix',
    'long_description': '# tracematrix\nA Python tool to create a traceability matrix.\n\n## Scope\nThis package focuses on generating the traceability matrix itself.\nAs the APIs and export formats of different test management and/or requirement management tools can be very different, the data acquisition and conversion is not in the scope of this package. However, it aims to provide a convenient way to create the individual items (e.g. requirements, testcases or any other traceable item) and traces between them.\n\n## How to use this package\nCurrently it is only possible to use this package programmatically in your own script.\n\nAll traceable items have a unique ``id`` and a set of other items that they are traced to.\n\nTo get an existing or create a new item, you can use the class method ``get_by_id(id_)``.\nIt will only create a new instance if no existing item with this ``id`` could be found.\nThis means that you don\'t have to keep track if you already processed this item, because\nsome items could appear more than once in your data (i.e. a requirement could appear on multiple test cases).\n\n```python\nfrom tracematrix.item import TraceItem\n\nreq1 = TraceItem.get_by_id("REQ_1")\ntestcase1 = TraceItem.get_by_id("TC_1")\n```\n\nCreating links between to items is done by simply passing the two items to ``TraceItem.add_trace(first, second)``.\nThis will create a bidirectional link between these elements and update the ``traced_to`` attribute on both.\n\n```python\nTraceItem.add_trace(req1, testcase1)\n```\n\nCurrently two output formats are supported - CSV and HTML.\nDefault is CSV, but you can specify the reporter when creating the ``TraceabilityMatrix``:\n\n```python\nfrom tracematrix.reporters import HtmlReporter\n\nmatrix = TraceabilityMatrix(testcases, requirements, reporter=HtmlReporter)\nmatrix.create_matrix("RequirementsTraceabilityMatrix.html")\n```\n',
    'author': 'Andreas Finkler',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6.2,<4.0.0',
}


setup(**setup_kwargs)



from typing import List

from tracematrix.item import TraceItem
from tracematrix.reporters import CsvReporter, Reporter


class TraceabilityMatrix:
    def __init__(
        self,
        testcases: List[TraceItem],
        requirements: List[TraceItem],
        reporter: Reporter = CsvReporter,
    ):
        self.testcases = testcases
        self.requirements = requirements
        self.reporter = reporter()

    def write_matrix(self, outputfile):
        self.reporter.write(outputfile, self.testcases, self.requirements)



from abc import ABC, abstractstaticmethod
from typing import List

from tracematrix.item import TraceItem


class Reporter(ABC):
    @abstractstaticmethod
    def write(
        outputfile: str, testcases: List[TraceItem], requirements: List[TraceItem]
    ) -> None:
        pass
